@@ -50,6 +50,7 @@ CREDITS = dict(
         'Giovanne Menicheli (Brazilian Portuguese)',
         'Fúlvio Alves (Brazilian Portuguese)',
         'Felipe (Brazilian Portuguese)',
+        'Matheus Santana (Brazilian Portuguese)',
         'twlvnn (Bulgarian)',
         'Roger VC (Catalan)',
         'Lukáš Linhart (Czech)',
@@ -82,6 +83,7 @@ CREDITS = dict(
         'Sear Gasor (German)',
         'Vortex Acherontic (German)',
         'Dlurak (German)',
+        'Mirko P. (German)',
         'Scrambled777 (Hindi)',
         'mondstern (Hungarian)',
         'Milo Ivir (Croatian)',
@@ -94,6 +96,7 @@ CREDITS = dict(
         'Davide Mora (Italian)',
         'Andrea Scarano (Italian)',
         'pasquale ruotolo (Italian)',
+        'Riccardo Luise (Italian)',
         'Velyvis (Lithuanian)',
         'Lili Kurek (Polish)',
         'Aleksander Warzyniak (Polish)',
@@ -101,6 +104,7 @@ CREDITS = dict(
         'Ademario Cunha (Portuguese)',
         'SpiralPack 527 (Portuguese)',
         'Lucas Silva Goulart (Portuguese)',
+        'Manuela Silva (Portuguese)',
         'shima (Russian)',
         'Valentin Chernetsov (Russian)',
         'FIONover (Russian)',
@@ -120,6 +124,7 @@ CREDITS = dict(
         'mondstern (Ukrainian)',
         'DXCVII (Ukrainian)',
         'Bezruchenko Simon (Ukrainian)',
+        'Максим Горпиніч (Ukrainian)',
         'niyaki hayyashi (Vietnamese)',
     ),
     supporters=(
@@ -130,12 +135,15 @@ CREDITS = dict(
 
 RELEASE_NOTES = """
 <ul>
-    <li>[Servers] Added Annyo (FR)</li>
-    <li>[Servers] Added Jen Sorensen Weekly Cartoon (EN)</li>
-    <li>[Servers] Added SushiScan.net (FR)</li>
-    <li>[Servers] EZManga (EN): Update</li>
-    <li>[Servers] Reaper Scans (EN): Disabled</li>
-    <li>[L10n] Updated German, Italian, Russian and Ukrainian translations</li>
+    <li>[UX] Refined visuals taking advantage of the new capabilities of GNOME 49</li>
+    <li>[UI] Switching to Glycin (image loading framework)</li>
+    <li>[Reader] Webtoon pager: Fixed small gap between pages</li>
+    <li>[Servers] EZmanga (EN): Update</li>
+    <li>[Servers] Grise Bouille (FR): Update</li>
+    <li>[Servers] MangaCrab (ES): Update</li>
+    <li>[Servers] Neko Scans (ES): Disabled</li>
+    <li>[Servers] Qi Scans (Night scans): Update</li>
+    <li>[L10n] Updated French, Portuguese, Portuguese (Brazil) and Ukrainian translations</li>
 </ul>
 <p>Happy reading.</p>
 """
