@@ -11,11 +11,11 @@ from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 
+from komikku.consts import MISSING_IMG_RESOURCE_PATH
 from komikku.models import Chapter
 from komikku.models import create_db_connection
 from komikku.utils import CoverPicture
 from komikku.utils import html_escape
-from komikku.utils import MISSING_IMG_RESOURCE_PATH
 
 DAYS_LIMIT = 30
 THUMB_WIDTH = 45
@@ -86,7 +86,7 @@ class HistoryDateChapterRow(Adw.ActionRow):
         # Time
         last_read = chapter.last_read.replace(tzinfo=pytz.UTC).astimezone(TIMEZONE)
         label = Gtk.Label(label=last_read.strftime('%H:%M'))
-        label.set_css_classes(['subtitle', 'monospace'])
+        label.set_css_classes(['subtitle', 'numeric'])
         self.add_suffix(label)
 
         # Play/Resume button
