@@ -5,13 +5,15 @@
 # Hean CMS
 
 # Supported servers:
+# EZmanga [EN]
 # Mode Scanlator [pt_BR] (disabled)
+# Night scans [EN]
 # Reaper Scans [EN] (disabled)
 # Reaper Scans [pt_BR] (disabled)
 
+
 import json
 import logging
-import re
 import time
 
 from bs4 import BeautifulSoup
@@ -59,176 +61,84 @@ def extract_info_from_script(soup, keyword):
 class HeanCMS(Server):
     base_url: str
     api_url: str
-    api_version: int
+
+    logo_url: str = None
     manga_url: str = None
     chapter_url: str = None
+    api_manga_url: str = None
     api_chapters_url: str = None
-    media_url: str = None
-
-    date_format = '%m/%d/%Y'
-    re_serie_id = r'.*{"series_id":(\d*).*'
-
-    name_css_path: str
-    cover_css_path: str
-    authors_css_path: str
-    synopsis_css_path: str
 
     def __init__(self):
+        if self.logo_url is None:
+            self.logo_url = self.base_url + '/favicon.ico'
         if self.manga_url is None:
             self.manga_url = self.base_url + '/series/{0}'
         if self.chapter_url is None:
             self.chapter_url = self.base_url + '/series/{0}/{1}'
+        if self.api_manga_url is None:
+            self.api_manga_url = self.api_url + '/post?postSlug={0}'
         if self.api_chapters_url is None:
-            self.api_chapters_url = self.api_url + '/chapter/query'
+            self.api_chapters_url = self.api_url + '/chapters?postId={0}&skip={1}&take={2}&order=desc&search=&userId=undefined'
 
         if self.session is None and not self.has_cf:
             self.session = requests.Session()
             self.session.headers.update({'User-Agent': USER_AGENT})
 
-    @staticmethod
-    def extract_chapter_nums_from_slug(slug):
-        """Extract chapter num from slug
-
-        Volume num is not available
-        """
-        re_nums = r'^((\w+)-)?(\d+)(-(\d+))?.*'
-
-        if matches := re.search(re_nums, slug):
-            if num := matches.group(3):
-                num = f'{int(num)}'
-
-                if num_dec := matches.group(5):
-                    num = f'{num}.{int(num_dec)}'
-
-                return num, None
-
-        return None, None
-
     @CompleteChallenge()
     def get_manga_data(self, initial_data):
         """
-        Returns manga data by scraping manga HTML page content
+        Returns manga data via API request
 
         Initial data should contain at least manga's slug (provided by search)
         """
         assert 'slug' in initial_data, 'Manga slug is missing in initial data'
 
-        r = self.session_get(self.manga_url.format(initial_data['slug']))
+        r = self.session_get(self.api_manga_url.format(initial_data['slug']))
         if r.status_code != 200:
             return None
 
-        mime_type = get_buffer_mime_type(r.content)
-        if mime_type != 'text/html':
-            return None
-
-        soup = BeautifulSoup(r.text, 'lxml')
-
-        if info := extract_info_from_script(soup, 'series_slug'):
-            # Extract serie_id
-            if matches := re.search(self.re_serie_id, info):
-                serie_id = matches.group(1)
-            else:
-                return None
+        resp_data = r.json()['post']
 
         data = initial_data.copy()
         data.update(dict(
+            name=resp_data['postTitle'],
             authors=[],
-            scanlators=[self.name, ],
+            scanlators=[],
             genres=[],
             status='ongoing',
             synopsis=None,
             chapters=[],
             server_id=self.id,
-            cover=None,
+            cover=resp_data['featuredImage'],
         ))
 
-        data['name'] = soup.find(self.name_css_path).text.strip()
-        if img_element := soup.select_one(self.cover_css_path):
-            data['cover'] = img_element.get('src')
-            if not data['cover'].startswith('http'):
-                if self.media_url:
-                    data['cover'] = self.media_url + data['cover']
-                else:
-                    data['cover'] = self.base_url + data['cover']
+        for genre in resp_data['genres']:
+            data['genres'].append(genre['name'])
 
-        # Details
-        if element := soup.select_one(self.authors_css_path):
-            for author in element.text.split('|'):
+        if resp_data['seriesStatus'] == 'COMPLETED':
+            data['status'] = 'complete'
+        elif resp_data['seriesStatus'] == 'ONGOING':
+            data['status'] = 'ongoing'
+        elif resp_data['seriesStatus'] == 'DROPPED':
+            data['status'] = 'suspended'
+        elif resp_data['seriesStatus'] == 'HIATUS':
+            data['status'] = 'hiatus'
+
+        if resp_data.get('author'):
+            for author in resp_data['author'].split(','):
                 data['authors'].append(author.strip())
+        if resp_data.get('artist'):
+            for artist in resp_data['artist'].split(','):
+                data['authors'].append(artist.strip())
 
-        data['synopsis'] = soup.select_one(self.synopsis_css_path).text.strip()
+        if synopsis := resp_data['postContent']:
+            data['synopsis'] = BeautifulSoup(synopsis, 'lxml').text.strip()
 
         # Chapters
-        data['chapters'] = self.get_manga_chapters_data(serie_id)
+        data['chapters'] = self.get_manga_chapters_data(resp_data['id'])
 
         return data
 
-    def get_manga_chapters_data(self, serie_id):
-        """
-        Returns manga chapters list via API
-        """
-        chapters = []
-
-        def get_page(serie_id, page):
-            if self.api_version == 1:
-                r = self.session_get(
-                    self.api_chapters_url,
-                    params=dict(
-                        page=page,
-                        perPage=100,
-                        series_id=serie_id,
-                    )
-                )
-            elif self.api_version == 2:
-                r = self.session_get(
-                    self.api_chapters_url.format(serie_id),
-                    params=dict(
-                        page=page,
-                        perPage=100,
-                    )
-                )
-            if r.status_code != 200:
-                return None, False, None
-
-            data = r.json()
-            if not data.get('data'):
-                return None, False, None
-
-            more = data.get('meta') and data['meta']['current_page'] != data['meta']['last_page']
-
-            return data['data'], more, get_response_elapsed(r)
-
-        chapters = []
-        delay = None
-        more = True
-        page = 1
-        while more:
-            if delay:
-                time.sleep(delay)
-
-            chapters_page, more, rtime = get_page(serie_id, page)
-            if chapters_page:
-                for chapter in chapters_page:
-                    if chapter['price'] > 0:
-                        continue
-
-                    num, _num_volume = self.extract_chapter_nums_from_slug(chapter['chapter_slug'])
-
-                    chapters.append(dict(
-                        slug=chapter['chapter_slug'],
-                        title=chapter['chapter_name'],
-                        num=num,
-                        date=convert_date_string(chapter['created_at'].split('T')[0], self.date_format) if 'created_at' in chapter else None,
-                    ))
-                page += 1
-                delay = min(rtime * 2, DOWNLOAD_MAX_DELAY) if rtime else None
-
-            elif chapters_page is None:
-                # Failed to retrieve a chapters list page, abort
-                break
-
-        return list(reversed(chapters))
-
     @CompleteChallenge()
     def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
         """
@@ -253,20 +163,74 @@ class HeanCMS(Server):
 
         if info := extract_info_from_script(soup, 'API_Response'):
             info = json.loads(info)
+            images = info[1][3]['API_Response']['chapter']['images']
 
             data = dict(
                 pages=[],
             )
-            for url in info[1][3]['children'][1][3]['children'][1][3]['API_Response']['chapter']['chapter_data']['images']:
+            for image in images:
                 data['pages'].append(dict(
                     slug=None,
-                    image=url,
+                    image=image['url'],
                 ))
 
             return data
 
         return None
 
+    def get_manga_chapters_data(self, serie_id):
+        """
+        Returns manga chapters list via API
+        """
+        chapters = []
+
+        def get_page(serie_id, page):
+            r = self.session_get(
+                self.api_chapters_url.format(serie_id, (page - 1) * 50, page * 50),
+                headers={
+                    'Referer': f'{self.base_url}/',
+                }
+            )
+            if r.status_code != 200:
+                return None, False, None
+
+            data = r.json()
+            if not data.get('post'):
+                return None, False, None
+
+            more = data['totalChapterCount'] > page * 50
+
+            return data['post']['chapters'], more, get_response_elapsed(r)
+
+        chapters = []
+        delay = None
+        more = True
+        page = 1
+        while more:
+            if delay:
+                time.sleep(delay)
+
+            chapters_page, more, rtime = get_page(serie_id, page)
+            if chapters_page:
+                for chapter in chapters_page:
+                    if chapter['price'] > 0:
+                        continue
+
+                    chapters.append(dict(
+                        slug=chapter['slug'],
+                        title=chapter['title'] or f'Chapter {chapter["number"]}',
+                        num=chapter['number'],
+                        date=convert_date_string(chapter['createdAt'].split('T')[0], '%Y-%m-%d') if 'createdAt' in chapter else None,
+                    ))
+                page += 1
+                delay = min(rtime * 2, DOWNLOAD_MAX_DELAY) if rtime else None
+
+            elif chapters_page is None:
+                # Failed to retrieve a chapters list page, abort
+                break
+
+        return list(reversed(chapters))
+
     @CompleteChallenge()
     def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
         """
@@ -306,24 +270,16 @@ class HeanCMS(Server):
 
     def get_manga_list(self, term=None, orderby=None):
         params = dict(
-            adult='true',
-            series_type='Comic',
-            status='All',
-            tags_ids='[]',
+            page=1,
+            perPage=21,
         )
         if term:
-            params['query_string'] = term
+            params['searchTerm'] = term
         else:
-            params.update(dict(
-                visibility='Public',
-                order='desc',
-                page=1,
-                perPage=20,
-            ))
             if orderby == 'latest':
-                params['orderBy'] = 'latest'
+                params['orderBy'] = 'updatedAt'
             elif orderby == 'popular':
-                params['orderBy'] = 'total_views'
+                params['orderBy'] = 'totalViews'
 
         r = self.session_get(
             self.api_url + '/query',
@@ -338,19 +294,14 @@ class HeanCMS(Server):
             return None
 
         results = []
-        for item in r.json()['data']:
-            cover = item['thumbnail']
-            if not cover.startswith('http'):
-                if self.media_url:
-                    cover = f'{self.media_url}/{cover}'
-                else:
-                    cover = f'{self.base_url}/{cover}'
+        for item in r.json()['posts']:
+            cover = item['featuredImage']
 
             results.append(dict(
-                slug=item['series_slug'],
-                name=item['title'],
+                slug=item['slug'],
+                name=item['postTitle'],
                 cover=cover,
-                last_chapter=item['chapters'][0]['chapter_name'] if item.get('chapters') else None,
+                last_chapter=item['chapters'][0]['number'] if item.get('chapters') else None,
             ))
 
         return results
