@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from komikku.servers.multi.manga_stream import MangaStream
@@ -11,9 +11,10 @@ class Noromax(MangaStream):
     lang = 'id'
 
     name_re_sub = r'Bahasa Indonesia'
-    series_name = 'Komik'
+    series_name = 'manga'
 
-    base_url = 'https://noromax.my.id'
+    base_url = 'https://noromax01.my.id'
+    logo_url = 'https://i0.wp.com/noromax.my.id/wp-content/uploads/2023/04/cropped-Untitled-1-192x192.png'
 
     authors_selector = '.infox .fmed:-soup-contains("Artist") span, .infox .fmed:-soup-contains("Author") span'
     genres_selector = '.infox .mgen a'
